/* ========================================
 * Example of UART communication with PSoC using hyperterminal 
 * send control commands to change PWM duty cycle for 8 PWMs
 *
 * example of valid commands (each dollowed by either LF, CR or both):
 * 'A4096' - set channel A to half of max value
 * 'B0'    - set channel B to zero
 * 'X'     - set all channels to max value
 * 'Z'     - set ll channels to Zero
 *
 * ========================================
*/
#include "project.h"
#include <string.h>
#include <stdlib.h> //atoi
#include <stdio.h>  //spintf
#include <myUART.h> //UART communication


char strMsg1[64]; //output UART buffer


//buffer to hold application settings
typedef struct TParamBuffer{
    uint16 A, B, C; //compare settings for PWM_A, PWM_B, ..., PWM_H
} ParamBuffer; //settings
volatile ParamBuffer PB;     

// each control command has a structure defined by TRxBuffer in myUART.h:
// CMD (1byte) | value_string (up to 19 bytes long) | separator (\r\n)
// example of valid commands (each dollowed by either LF, CR or both):
// 'A4096' - set channel A to half of max value
// 'B0'    - set channel B to zero
// 'X'     - set all channels to max value
// 'Z'     - set ll channels to Zero

    
//define input commands-> 
#define cmd_A  'A' //set value for PWM_A
#define cmd_B  'B' //set value for PWM_B
#define cmd_C  'C' //set value for PWM_C
    
#define cmd_X  'X' //set all PWMs to MaxVal
#define cmd_Z  'Z' //set all PWMs to Zero   

const uint16 MaxVal = 8191; // (2^13-1) max PWM value, resulting PWM frequency = Clock_PWM / MaxVal




//=======================================
//         Forward declarations  
//=======================================
void ProcessCommandMsg(void);      // Process received command (do something...)
void RefreshPWM(void);             // refresh all PWM values
void SetPWMperiod(void);           // set common period for all PWMs (PWM freq = Clock_PWM/MaxVal)
int  isNumeric(const char * s);    // check if string is a number



void Initialize(void)
{
	CyGlobalIntEnable;
    
	Rx_Int_StartEx(MyRxInt);//start Rx interrupt
    //no Tx interrupt - use large Tx buffer size, which disables Tx interrupt
    	
	UART_Start();
    CyDelay(250);
    UART_PutString("\r\nPWM + UART example\r\n");
    UART_PutString("type a command, e.g. A255 >\r\n");
    
   
     
    PWM_A_Start(); PWM_B_Start(); PWM_C_Start();
    PB.A = PB.B = PB.C = 0;
    RefreshPWM();
    SetPWMperiod();
    
}

int main()
{   
    Initialize();
    
	for(;;) //forever 
	{   
        
        if(IsCharReady()) //Rx buffer not empty
        {
            if (GetRxStr())   //extract first command message (non-blocking)
            {
                ProcessCommandMsg(); 
            }
        }               
		      
        
    }
}


//===========================================================================
// Process UART Receive Buffer content: RB.RxStr = RB.cms + RB.valstr
// 
//===========================================================================

void ProcessCommandMsg(void)
{    
    // check received message for any valid command and execute it if necessary or report old value
    // if command not recognized, then report error (!)
    // todo: current version does not check for valid input range

    
    if  (RB.cmd == cmd_A)//command 'A' received..
    {
        if (isNumeric(RB.valstr)) PB.A = atoi(RB.valstr);//if valid number set new value
        sprintf(strMsg1,"A=%d\r\n", PB.A); UART_PutString(strMsg1);//echo new value
        PWM_A_WriteCompare(PB.A); //update settings
    }
    else if (RB.cmd == cmd_B)//command 'B' received..
    {
        if (isNumeric(RB.valstr)) PB.B = atoi(RB.valstr);//if valid number set new value, else report old 
        sprintf(strMsg1,"B=%d\r\n", PB.B); UART_PutString(strMsg1);//echo command and value
        PWM_B_WriteCompare(PB.B);
    }
    else if (RB.cmd == cmd_C)//command 'C' received..
    {
        if (strlen(RB.valstr) > 0) PB.C = atoi(RB.valstr);//set new value, else report old 
        sprintf(strMsg1,"C=%d\r\n", PB.C); UART_PutString(strMsg1);//echo command and value
        PWM_C_WriteCompare(PB.C);
    }
    else if (RB.cmd == cmd_X)//command 'X' received..
    {
        sprintf(strMsg1,"X=%s\r\n", RB.valstr); UART_PutString(strMsg1);//echo command and value
        PB.A = PB.B = PB.C = MaxVal;
        RefreshPWM(); // set all to max value
    }
    else if (RB.cmd == cmd_Z)//command 'Z' received..
    {
        sprintf(strMsg1,"Z=%s\r\n", RB.valstr); UART_PutString(strMsg1);//echo command and value
        PB.A = PB.B = PB.C = 0;
        RefreshPWM(); // turn off all 
    }
    else //command unrecognized - echo unrecognized command
    {
        sprintf(strMsg1,"!%s\r\n", RB.RxStr); UART_PutString(strMsg1);
    }
  
}

//===========================================================================
// Update all PWMs values
//===========================================================================

void RefreshPWM(void)
{
    PWM_A_WriteCompare(PB.A);
    PWM_B_WriteCompare(PB.B);
    PWM_C_WriteCompare(PB.C);
}

//===========================================================================
// Update all PWMs values
//===========================================================================

void SetPWMperiod(void)
{
    PWM_A_WritePeriod(MaxVal);
    PWM_B_WritePeriod(MaxVal);
    PWM_C_WritePeriod(MaxVal);
}

//===========================================================================
// Check that string represents a number
// function does not checksa bounds, and input string must be trimmed
//
// URL: 
// http://rosettacode.org/wiki/Determine_if_a_string_is_numeric#C
//===========================================================================
// todo: check input range
int isNumeric (const char * s)
{
    if (s == NULL || *s == '\0' || isspace(*s))
      return 0;
    char * p;
    strtod (s, &p);
    return *p == '\0';
}

/* [] END OF FILE */
